package App::Beam::History::Entry;

use strict;
use Carp;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(STATE_PENDING STATE_SUCCESS STATE_FAILURE state_string);
our %EXPORT_TAGS = (
    state => [ qw(STATE_PENDING STATE_SUCCESS STATE_FAILURE) ]
);
 
use constant {
    STATE_PENDING => 0,
    STATE_SUCCESS => 1,
    STATE_FAILURE => 2
};

my @state_str = (
    'PENDING',
    'SUCCESS',
    'FAILURE'
);

sub new {
    my $class = shift;
    return  bless { start_time => time(),
		    end_time => time(),
		    state => STATE_PENDING }, $class;
}

sub state {
    my ($self, $value) = @_;
    if (defined($value)) {
	croak "can't modify finished entry"
	    unless $self->{state} == STATE_PENDING;
	$self->{state} = $value;
	$self->{end_time} = time();
    }
    return $self->{state};
}

sub state_string {
    my ($state) = @_;
    return $state unless $state <= $#state_str;
    return $state_str[$state];
}

sub state_name {
    my ($self) = @_;
    return state_string($self->state);
}

sub start_time {
    my ($self) = @_;
    return $self->{start_time};
}

sub end_time {
    my ($self) = @_;
    return $self->{end_time};
}

1;



   
