package App::Beam::History::Record;

use strict;
use Carp;
use App::Beam::History::Entry qw(:state);
use App::Beam::History::Item;

our @ISA = qw(App::Beam::History::Entry);

sub new {
    my $class = shift;
    my $self = $class->SUPER::new;
    $self->{detail} = {};
    
    local %_ = @_;
    my $v;

    if (defined($v = delete($_{prev}))) {
	my ($maxround, $maxlevel);
	unless (defined($maxround = delete($_{rounds}))) {
	    croak "'prev' requires parameter 'rounds', which is not given";
	}
	unless (defined($maxlevel = delete($_{levels}))) {
	    croak "'prev' requires parameter 'levels', which is not given";
	}
#	print "PREV ".$v->level."\n";
	($self->{level}, $self->{round}, $self->{cycle}) =
	    ($v->level, $v->round, $v->cycle);
	$self->{level}++;
	if ($self->{level} > $maxlevel) {
	    $self->{level} = 0;
	    $self->{round}++;
	    if ($self->{round} > $maxround) {
		$self->{round} = 0;
		$self->{cycle}++;
	    }
	}
    } else {
	if (defined($v = delete($_{cycle}))) {
	    $self->{cycle} = $v;
	} else {
	    $self->{cycle} = 0;
	}

	if (defined($v = delete($_{round}))) {
	    $self->{round} = $v;
	} else {
	    $self->{round} = 0;
	}

	if (defined($v = delete($_{level}))) {
	    $self->{level} = $v;
	} else {
	    $self->{level} = 0;
	}
    }
    
    if (keys(%_)) {
	croak "unknown or unhandled parameters: ".join(',', keys(%_));
    }

    return $self;
}

sub reset {
    my $self = shift;
    delete @{$self->{detail}}{@_} if (@_);
    $self->{state} = STATE_PENDING;
}

sub cycle {
    my ($self) = @_;
    return $self->{cycle};
}

sub round {
    my ($self) = @_;
    return $self->{round};
}

sub level {
    my ($self) = @_;
    return $self->{level};
}

sub state {
    my ($self, $item) = @_;
    if (defined($item)) {
	return undef unless exists $self->{detail}{$item};
	return $self->{detail}{$item}->state;
    }
    return $self->{state};
}

sub start_time {
    my ($self, $item) = @_;
    if (defined($item)) {
	return undef unless exists $self->{detail}{$item};
	return $self->{detail}{$item}->start_time;
    }
    return $self->SUPER::start_time;
}

sub end_time {
    my ($self, $item) = @_;
    if (defined($item)) {
	return undef unless exists $self->{detail}{$item};
	return $self->{detail}{$item}->end_time;
    }
    return $self->SUPER::end_time;
}

sub begin_item {
    my ($self, $name) = @_;
    croak "can't add item to a finished record"
	unless $self->{state} == STATE_PENDING;
    my $item = new App::Beam::History::Item;
    $self->{detail}{$name} = $item;
    if ($item->start_time > $self->{end_time}) {
	$self->{end_time} = $item->start_time;
    }
    return $item;
}

sub items {
    my ($self) = @_;
    return %{$self->{detail}};
}

sub item_names {
    my ($self) = @_;
    return keys %{$self->{detail}};
}

sub finish_item {
    my ($self, $name, $state) = @_;
    croak "can't modify item in a finished record"
	unless $self->{state} == STATE_PENDING;
    croak "no such item"
	unless exists $self->{detail}{$name};
    my $item = $self->{detail}{$name};
    $item->state($state);
    if ($item->end_time > $self->{end_time}) {
	$self->{end_time} = $item->end_time;
    }
}

sub finish {
    my ($self) = @_;
    croak "can't finish a finished record"
	unless $self->{state} == STATE_PENDING;
    my @x = map {$_->state} values %{$self->{detail}};
    if (@x) {
	$self->{state} = (grep { $_ != STATE_SUCCESS } @x)
	                  ? STATE_FAILURE
                          : STATE_SUCCESS;
    }
    return $self->{state};
}

1;





